@@ -22,10 +22,16 @@ import org.apache.kafka.common.Cluster;
 import org.apache.kafka.common.Node;
 import org.apache.kafka.common.PartitionInfo;
 import org.apache.kafka.common.TopicPartition;
+import org.apache.kafka.common.feature.Features;
+import org.apache.kafka.common.feature.SupportedVersionRange;
+import org.apache.kafka.common.message.ApiMessageType;
+import org.apache.kafka.common.message.ApiVersionsResponseData;
 import org.apache.kafka.common.network.NetworkReceive;
 import org.apache.kafka.common.network.Send;
 import org.apache.kafka.common.protocol.ApiKeys;
+import org.apache.kafka.common.record.RecordVersion;
 import org.apache.kafka.common.record.UnalignedRecords;
+import org.apache.kafka.common.requests.ApiVersionsResponse;
 import org.apache.kafka.common.requests.ByteBufferChannel;
 import org.apache.kafka.common.requests.RequestHeader;
 import org.apache.kafka.common.utils.Exit;
@@ -34,10 +40,10 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.File;
-import java.io.FileWriter;
 import java.io.IOException;
 import java.lang.reflect.Field;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.util.ArrayList;
@@ -141,15 +147,23 @@ public class TestUtils {
         return b.toString();
     }
 
+    /**
+     * Create an empty file in the default temporary-file directory, using the given prefix and suffix
+     * to generate its name.
+     * @throws IOException
+     */
+    public static File tempFile(final String prefix, final String suffix) throws IOException {
+        final File file = Files.createTempFile(prefix, suffix).toFile();
+        file.deleteOnExit();
+        return file;
+    }
+
     /**
      * Create an empty file in the default temporary-file directory, using `kafka` as the prefix and `tmp` as the
      * suffix to generate its name.
      */
     public static File tempFile() throws IOException {
-        final File file = File.createTempFile("kafka", ".tmp");
-        file.deleteOnExit();
-
-        return file;
+        return tempFile("kafka", ".tmp");
     }
 
     /**
@@ -158,10 +172,7 @@ public class TestUtils {
      */
     public static File tempFile(final String contents) throws IOException {
         final File file = tempFile();
-        final FileWriter writer = new FileWriter(file);
-        writer.write(contents);
-        writer.close();
-
+        Files.write(file.toPath(), contents.getBytes(StandardCharsets.UTF_8));
         return file;
     }
 
@@ -576,4 +587,57 @@ public class TestUtils {
 
         return allSegmentsSet.equals(expectedSegmentsSet);
     }
+
+    public static ApiVersionsResponse defaultApiVersionsResponse(
+            ApiMessageType.ListenerType listenerType
+    ) {
+        return defaultApiVersionsResponse(0, listenerType);
+    }
+
+    public static ApiVersionsResponse defaultApiVersionsResponse(
+            int throttleTimeMs,
+            ApiMessageType.ListenerType listenerType
+    ) {
+        return createApiVersionsResponse(
+                throttleTimeMs,
+                ApiVersionsResponse.filterApis(RecordVersion.current(), listenerType, true, true),
+                Features.emptySupportedFeatures(),
+                false
+        );
+    }
+
+    public static ApiVersionsResponse defaultApiVersionsResponse(
+            int throttleTimeMs,
+            ApiMessageType.ListenerType listenerType,
+            boolean enableUnstableLastVersion
+    ) {
+        return createApiVersionsResponse(
+                throttleTimeMs,
+                ApiVersionsResponse.filterApis(RecordVersion.current(), listenerType, enableUnstableLastVersion, true),
+                Features.emptySupportedFeatures(),
+                false
+        );
+    }
+
+    public static ApiVersionsResponse createApiVersionsResponse(
+            int throttleTimeMs,
+            ApiVersionsResponseData.ApiVersionCollection apiVersions
+    ) {
+        return createApiVersionsResponse(throttleTimeMs, apiVersions, Features.emptySupportedFeatures(), false);
+    }
+
+    public static ApiVersionsResponse createApiVersionsResponse(
+            int throttleTimeMs,
+            ApiVersionsResponseData.ApiVersionCollection apiVersions,
+            Features<SupportedVersionRange> latestSupportedFeatures,
+            boolean zkMigrationEnabled
+    ) {
+        return ApiVersionsResponse.createApiVersionsResponse(
+                throttleTimeMs,
+                apiVersions,
+                latestSupportedFeatures,
+                Collections.emptyMap(),
+                ApiVersionsResponse.UNKNOWN_FINALIZED_FEATURES_EPOCH,
+                zkMigrationEnabled);
+    }
 }
