@@ -78,6 +78,7 @@ import java.util.Set;
 import java.util.function.Consumer;
 import java.util.stream.Collectors;
 
+import static org.apache.kafka.raft.LeaderState.CHECK_QUORUM_TIMEOUT_FACTOR;
 import static org.apache.kafka.raft.RaftUtil.hasValidTopicPartition;
 import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
 import static org.junit.jupiter.api.Assertions.assertEquals;
@@ -91,6 +92,7 @@ public final class RaftClientTestContext {
     final int electionBackoffMaxMs = Builder.ELECTION_BACKOFF_MAX_MS;
     final int fetchMaxWaitMs = Builder.FETCH_MAX_WAIT_MS;
     final int fetchTimeoutMs = Builder.FETCH_TIMEOUT_MS;
+    final int checkQuorumTimeoutMs = (int) (fetchTimeoutMs * CHECK_QUORUM_TIMEOUT_FACTOR);
     final int retryBackoffMs = Builder.RETRY_BACKOFF_MS;
 
     private int electionTimeoutMs;
@@ -1116,6 +1118,10 @@ public final class RaftClientTestContext {
             return currentLeaderAndEpoch;
         }
 
+        List<Batch<String>> committedBatches() {
+            return commits;
+        }
+
         Batch<String> lastCommit() {
             if (commits.isEmpty()) {
                 return null;
@@ -1140,14 +1146,6 @@ public final class RaftClientTestContext {
             }
         }
 
-        List<String> commitWithBaseOffset(long baseOffset) {
-            return commits.stream()
-                .filter(batch -> batch.baseOffset() == baseOffset)
-                .findFirst()
-                .map(batch -> batch.records())
-                .orElse(null);
-        }
-
         List<String> commitWithLastOffset(long lastOffset) {
             return commits.stream()
                 .filter(batch -> batch.lastOffset() == lastOffset)
@@ -1194,14 +1192,14 @@ public final class RaftClientTestContext {
 
         @Override
         public void handleLeaderChange(LeaderAndEpoch leaderAndEpoch) {
-            // We record the next expected offset as the claimed epoch's start
+            // We record the current committed offset as the claimed epoch's start
             // offset. This is useful to verify that the `handleLeaderChange` callback
-            // was not received early.
+            // was not received early on the leader.
             this.currentLeaderAndEpoch = leaderAndEpoch;
 
             currentClaimedEpoch().ifPresent(claimedEpoch -> {
                 long claimedEpochStartOffset = lastCommitOffset().isPresent() ?
-                    lastCommitOffset().getAsLong() + 1 : 0L;
+                    lastCommitOffset().getAsLong() : 0L;
                 this.claimedEpochStartOffsets.put(leaderAndEpoch.epoch(), claimedEpochStartOffset);
             });
         }
@@ -1216,7 +1214,7 @@ public final class RaftClientTestContext {
         }
 
         @Override
-        public void handleSnapshot(SnapshotReader<String> reader) {
+        public void handleLoadSnapshot(SnapshotReader<String> reader) {
             snapshot.ifPresent(snapshot -> assertDoesNotThrow(snapshot::close));
             commits.clear();
             savedBatches.clear();
